@@ -1,6 +1,5 @@
 
-# Base class for Chronic.  For the moment, just contains 
-# a debug method. 
+# Base class for Chronic.  
 
 package Schedule::Chronic::Base; 
 
@@ -24,5 +23,21 @@ sub fatal {
 }
 
 
+sub which { 
+
+    my ($self, $app) = @_;
+
+    my $path = `which $app`;
+    chomp $path;
+
+    if ($path =~ /no $app/) { 
+        return;
+    } 
+
+    return $path;
+
+}
+
+
 1;
 
